from re import M
import sqlite3
from datetime import datetime
import click
from flask import current_app, g

def log(statement, user_id, before, after, change):
    db = get_db()
    c = db.cursor()
    c.execute("INSERT INTO transaction_log (timestamp, type, user_id, before, after, change) VALUES (?, ?, ?, ?, ?, ?)", [datetime.now(), statement, user_id, before, after, change])
    db.commit()

def change_db(statement, user_id=None, before=None, after=None, change=None):
    db = get_db()
    c = db.cursor()
    if statement == "adduser" and after != None:
        c.execute("INSERT or IGNORE INTO users (username, balance) VALUES (?, 0)", [after])
        user_id = c.lastrowid
    elif statement == "removeuser" and user_id != None and before != None:
        c.execute("DELETE FROM tags WHERE userid=?", [user_id])
        c.execute("DELETE FROM users WHERE id=?", [user_id])
    elif statement == "addtag" and after != None and user_id != None:
        c.execute("INSERT OR IGNORE INTO tags (tagid, userid) VALUES ?, ?)", [after, user_id])
    elif statement == "removetag" and before != None and user_id != None:
        c.execute("DELETE FROM tags WHERE (tagid = ? AND userid = ?)", [before, user_id])
    elif statement == "balance" and change != None and user_id != None:
        c.execute("UPDATE users SET balance = balance + ? WHERE id=?", [change, user_id])
    else:
        raise Exception("wrong or missing argument for change_db")
    log(statement, user_id, before, after, change)
    db.commit()

def get_db():
    if 'db' not in g:
        g.db = sqlite3.connect(
            current_app.config['DATABASE'],
            detect_types=sqlite3.PARSE_DECLTYPES
        )
        g.db.row_factory = sqlite3.Row
        init_db()
    return g.db


def close_db(e=None):
    db = g.pop('db', None)

    if db is not None:
        db.close()

def init_db():
    db = get_db()

    with current_app.open_resource('schema.sql') as f:
        db.executescript(f.read().decode('utf8'))


@click.command('init-db')
def init_db_command():
    """Clear the existing data and create new tables."""
    init_db()
    click.echo('Initialized the database.')
